-----------------------------------------------------------------------------------------------
-- Client Lua Script for ScanbotRAM
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
--[[
Notes:
Fieldstudy icons: search for fieldstudy with icons on
Scientist mission icons: search for scientist with icons on
]]--

require "Window"
 
-----------------------------------------------------------------------------------------------
-- ScanbotRAM Module Definition
-----------------------------------------------------------------------------------------------
local ScanbotRAM = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ScanbotRAM:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- Persistent variables
	o.tMissions = {}	-- Stores all the path missions

	-- Helper variables
	o.tStories = nil 	-- Stores the stories
	o.timerErase = nil 	-- Timer until stories are deleted
	o.wndSelected = nil -- Keeps track of what mission is selected
	o.bIsReady = false 	-- Flag used to fix race condition between OnRestore and OnDocLoaded

    return o
end

function ScanbotRAM:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ScanbotRAM OnLoad, OnSave and OnRestore
-----------------------------------------------------------------------------------------------
function ScanbotRAM:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("ScanbotRAM.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function ScanbotRAM:OnSave(eLevel)
	-- General = 1 save across all accounts, realms and character
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then
        return nil
    end

    return self.tMissions
end

function ScanbotRAM:OnRestore(eLevel, tData)
	if not tData then return end
	self.tMissions = tData
end


-----------------------------------------------------------------------------------------------
-- ScanbotRAM OnDocLoaded
-----------------------------------------------------------------------------------------------
function ScanbotRAM:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ScanbotRAMForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		
		-- Register handlers for events, slash commands and timer, etc.
		-- Event used to store any active path missions
		Apollo.RegisterEventHandler("PlayerPathMissionUpdate", "OnMissionUpdate", self)
		-- TODO: maybe use PlayerPath_NotificationSent instead?

		-- Event for the pop up dialog window that contains the scan stories
		Apollo.RegisterEventHandler("StoryPanelDialog_Show", "OnNewStory", self)
		-- Slash command event
		Apollo.RegisterSlashCommand("ram", "OnScanbotRAMOn", self)

		-- TODO: Check this event:
		-- Apollo.RegisterEventHandler("PlayerPathScientistScanData", "OnScientistScanData", self)
		-- OnScientisScanData(tScannedUnits) tScannedUnits[x]. 	nDisplayCount
		--														strName
		--														nReceived
		
		-- TODO: Find a way of catching Datachron messages
		-- Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		-- Apollo.RegisterEventHandler("CombatLogDatacube", "OnCombatLogDatacube", self)
		-- Apollo.RegisterEventHandler("LuaChatLogMessage", "OnLuaChatLogMessage", self)

		-- Timer used to erase cached stories
		-- TODO: Find a better way to determine if stories are ment for path or not
		-- Perhaps check dialog box type?
		self.timerErase = ApolloTimer.Create(10, false, "OnTimerErase", self)
		self.timerErase:Stop()
	end
end

-----------------------------------------------------------------------------------------------
-- ScanbotRAM event functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/ram"
function ScanbotRAM:OnScanbotRAMOn()
	self.wndMain:FindChild("ScrollList"):DestroyChildren() -- clean it up just in case
	self.wndMain:Invoke() -- show the window
	self:RedrawAllMissions() -- fill up the window
	Print("[ScanbotRAM] Opening main window.")
end

-- Triggered by event PlayerPathMissionUpdate
function ScanbotRAM:OnMissionUpdate(pmMission)
	if not pmMission or pmMission:GetName()=="" then return end
	local nId = pmMission:GetId()
	Print("[ScanbotRAM] Mission update: ")
	if not self.tMissions[nId] then
		Print("[ScanbotRAM]    - Adding new mission.")
		-- Gather all the info on the mission
		self.tMissions[nId] = {}
		self.tMissions[nId].tStories = {}
		self.tMissions[nId].nCount = 0
		self.tMissions[nId].sName = pmMission:GetName()
		self.tMissions[nId].sIcon = self:GetScientistIcon(pmMission)
		self.tMissions[nId].bNew = true
		self.tMissions[nId].nCurrent = pmMission:GetNumCompleted()
		self.tMissions[nId].nGoal = pmMission:GetNumNeeded()
		-- Get zone name through the path episode
		local pepCurrent = PlayerPathLib.GetCurrentEpisode()
		self.tMissions[nId].sZone = pepCurrent:GetWorldZone()

		-- Now add it to the GUI list
		self:DrawMission(nId)
		self.wndMain:FindChild("ScrollList"):ArrangeChildrenVert(0)
	elseif self.tMissions[nId].nCurrent < pmMission:GetNumCompleted() then
		-- update progress
		Print("[ScanbotRAM]    - Updating progress.")
		self.tMissions[nId].nCurrent = pmMission:GetNumCompleted()
		for idx, wndItem in pairs(self.wndMain:FindChild("ScrollList"):GetChildren()) do
			if wndItem:GetData() == nId then
				self:DrawProgress(wndItem,nId)
			end
		end
	end	

	if self.tMissions[nId].tStories and self.tStories then
		Print("[ScanbotRAM]    - Adding new stories.")
		for i=1, #self.tStories do
			-- TODO: this is probably bugged? possibly always returns here
			for idx, tText in pairs (self.tMissions[nId].tStories) do
				if tText[1] == self.tStories[i][1] then 
					Print("[ScanbotRAM] Attempt to add duplicate story to MISSION!")
					return
				end
			end
			-- END TODO
			table.insert(self.tMissions[nId].tStories, self.tStories[i])
			Print("[ScanbotRAM] Story added.")
		end
	end
end

-- Triggered by event StoryPanelDialog_Show
function ScanbotRAM:OnNewStory(eWindowType, tLines)
	local nIndex = 1
	if not self.tStories then 
		self.tStories = {}
	else
		nIndex = #self.tStories + 1 
	end

	Print("[ScanbotRAM] Story update: ")

	-- TODO: this is probably bugged? possibly always returns here
	for i=1, (nIndex-1) do
		if self.tStories[i] == tLines then 
			Print("[ScanbotRAM] Attempt to add duplicate story to CACHE!")
			return 
		end
	end
	-- END TODO

	self.tStories[nIndex] = tLines
	Print("[ScanbotRAM]    - Added story to cache, (re)starting timer.")

	self.timerErase:Stop()
	self.timerErase:Start()
end

-- Triggered by timer
function ScanbotRAM:OnTimerErase()
	self.tStories = nil
	self.timerErase:Stop()
	Print("[ScanbotRAM]    - Timer done, cleared cache.")
end

-----------------------------------------------------------------------------------------------
-- ScanbotRAM draw functions
-----------------------------------------------------------------------------------------------

function ScanbotRAM:DrawMission(nId)
	if not self.tMissions[nId] then return end
	local tMission = self.tMissions[nId]

	local wndList = self.wndMain:FindChild("ScrollList")
	local wndItem = Apollo.LoadForm(self.xmlDoc, "MissionItem", wndList, self)
	
	wndItem:SetData(nId)
	wndItem:FindChild("MissionTitle"):SetText(tMission.sName)
	wndItem:FindChild("MissionIcon"):SetSprite(tMission.sIcon)
	if tMission.bNew then wndItem:FindChild("NewIndicator"):Show(true,true) end
	wndItem:FindChild("MissionProgress"):SetMax(tMission.nGoal)
	self:DrawProgress(wndItem,nId)
end

function ScanbotRAM:DrawProgress(wndItem,nId)
	local tMission = self.tMissions[nId]
	wndItem:FindChild("MissionProgressText"):SetText("Stories unlocked: "..tMission.nCurrent.."")
	wndItem:FindChild("MissionProgress"):SetProgress(tMission.nCurrent)
end


function ScanbotRAM:RedrawAllMissions()
	local wndList = self.wndMain:FindChild("ScrollList")

	for idx, tMission in pairs(self.tMissions) do
		self:DrawMission(idx)
	end
	wndList:ArrangeChildrenVert(0)
end

function ScanbotRAM:DrawArticle(nId)
	local wndArticle = Apollo.LoadForm(self.xmlDoc, "ArticleDisplay", self.wndMain:FindChild("ListBG"), self)
	local tMission = self.tMissions[nId]

	wndArticle:FindChild("ArticleTitle"):SetText(tMission.sName)
	wndArticle:FindChild("ArticleDisplayIcon"):SetSprite(tMission.sIcon)
end

function ScanbotRAM:DrawStories(nId)
	local tMission = self.tMissions[nId]

	if tMission.tStories then
		for idx, tText in pairs (tMission.tStories) do
			local wndStory = Apollo.LoadForm(self.xmlDoc, "StoryItem", self.wndMain:FindChild("ArticleScroll"), self)
			wndStory:FindChild("StoryTitle"):SetText("Entry " .. idx .. "")	
			local strText = ""
			for i=1, #tText do
				strText = strText .. tText[i] .. "\n"
			end
			wndStory:FindChild("StoryText"):SetText(strText)	
			wndStory:FindChild("StoryText"):SetHeightToContentHeight()
			wndStory:FindChild("DeleteStoryButton"):SetData({nMissionId = nId, nStoryId = idx})

			--wndStory:Show(true,true)

			local nTextLeft, nTextTop, nTextRight, nTextBottom = wndStory:FindChild("StoryText"):GetAnchorOffsets()
			local nLeft, nTop, nRight, nBottom = wndStory:GetAnchorOffsets()
			wndStory:SetAnchorOffsets(nLeft, nTop, nRight, nTextBottom + 8)
		end
	end
	self.wndMain:FindChild("ArticleScroll"):ArrangeChildrenVert(0)
end

-----------------------------------------------------------------------------------------------
-- ScanbotRAM - ScanbotRAMForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function ScanbotRAM:OnCancel()
	self.wndMain:Close() -- hide the window
	self.wndMain:FindChild("ScrollList"):DestroyChildren() -- clean it up
	Print("[ScanbotRAM] Closed main window.")
end

function ScanbotRAM:OnMissionItemDown(wndHandler, wndControl, eMouseButton)
	if not eMouseButton == GameLib.CodeEnumInputMouse.Left then return end
	local nMissionId = wndControl:GetData()
	Print("[ScanbotRAM] Clicked on mission " .. nMissionId .. ", opening Article.")
	wndControl:FindChild("NewIndicator"):Show(false,false)
	self.tMissions[nMissionId].bNew = false
	self:DrawArticle(nMissionId)
	self:DrawStories(nMissionId)
end

-----------------------------------------------------------------------------------------------
-- ScanbotRAM - ArticleDisplay Functions
-----------------------------------------------------------------------------------------------

function ScanbotRAM:OnBack(wndHandler, wndControl)
	local wndContainer = wndControl:GetParent():GetParent()
	wndContainer:DestroyChildren() -- KILLKILLKILL!
	wndContainer:Close() -- hide the window
	Print("[ScanbotRAM] Closed Article.")
end

---------------------------------------------------------------------------------------------------
-- ScanbotRAM - StoryItem Functions
---------------------------------------------------------------------------------------------------

function ScanbotRAM:OnDeleteStoryButtonDown( wndHandler, wndControl, eMouseButton)
	if not eMouseButton == GameLib.CodeEnumInputMouse.Left then return end
	local wndDialog = Apollo.LoadForm(self.xmlDoc, "ConfirmDeletion", nil, self)
	self.wndSelected = wndControl
	Print("[ScanbotRAM] Open ConfirmDeletion.")
end

---------------------------------------------------------------------------------------------------
-- ScanbotRAM - ConfirmDeletion Functions
---------------------------------------------------------------------------------------------------

function ScanbotRAM:OnDeleteButton( wndHandler, wndControl, eMouseButton )
	if not eMouseButton == GameLib.CodeEnumInputMouse.Left then return end
	local tTarget=self.wndSelected:GetData()
	Print("[ScanbotRAM] Deleted story " .. tTarget.nStoryId .. " from mission " .. tTarget.nMissionId .. ".")
	table.remove(self.tMissions[tTarget.nMissionId].tStories, tTarget.nStoryId)

	self.wndSelected:GetParent():Close()
	self.wndMain:FindChild("ArticleScroll"):ArrangeChildrenVert(0)

	wndControl:GetParent():Close()
end

function ScanbotRAM:OnCancelButton( wndHandler, wndControl, eMouseButton )
	if not eMouseButton == GameLib.CodeEnumInputMouse.Left then return end
	Print("[ScanbotRAM] Cancelled deletion.")
	wndControl:GetParent():Close()
end

-----------------------------------------------------------------------------------------------
-- ScanbotRAM helper functions
-----------------------------------------------------------------------------------------------

function ScanbotRAM:GetScientistIcon(pmMission)
	-- Icon Type -- taken from PathScientistContent -- looks deprecated, so might get broken by patch
	local strScientistIcon = pmMission:GetScientistIcon()
	local eType = pmMission:GetType()
	
	if eType == PathMission.PathMissionType_Scientist_Scan or eType == PathMission.PathMissionType_Scientist_ScanChecklist then
		local eSubType = pmMission:GetSubType()
		if eSubType == PathMission.ScientistCreatureType_Tech then
			strScientistIcon = "Icon_Mission_Scientist_ScanTech"
		elseif eSubType == PathMission.ScientistCreatureType_Flora then
			strScientistIcon = "Icon_Mission_Scientist_ScanPlant"
		elseif eSubType == PathMission.ScientistCreatureType_Fauna then
			strScientistIcon = "Icon_Mission_Scientist_ScanCreature"
		elseif eSubType == PathMission.ScientistCreatureType_Mineral then
			strScientistIcon = "Icon_Mission_Scientist_ScanMineral"
		end

	elseif eType == PathMission.PathMissionType_Scientist_FieldStudy then
		strScientistIcon = "Icon_Mission_Scientist_FieldStudy"
	elseif eType == PathMission.PathMissionType_Scientist_DatacubeDiscovery then
		strScientistIcon = "Icon_Mission_Scientist_DatachronDiscovery"
	elseif eType == PathMission.PathMissionType_Scientist_SpecimenSurvey then
		strScientistIcon = "Icon_Mission_Scientist_SpecimenSurvey"
	elseif eType == PathMission.PathMissionType_Scientist_Experimentation then
		strScientistIcon = "Icon_Mission_Scientist_ReverseEngineering"
	end

	return strScientistIcon
end


-----------------------------------------------------------------------------------------------
-- ScanbotRAM Instance
-----------------------------------------------------------------------------------------------
local ScanbotRAMInst = ScanbotRAM:new()
ScanbotRAMInst:Init()
